unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Spin, ComCtrls, Clipbrd, Mask;

type
  TfrmMain = class(TForm)
    // labels...
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    lblResult: TLabel;
    lblRound: TLabel;
    // panels...
    Panel1: TPanel;
    // edit boxes...
    edtST: TEdit;
    edtAWS: TEdit;
    edtTmp: TEdit;
    edtWS: TEdit;
    edtC1: TEdit;
    edtC2: TEdit;
    edtRound: TMaskEdit;
    // up-down buttons...
    ud1: TUpDown;
    ud2: TUpDown;
    udRound: TUpDown;
    // radio button groups...
    rgTS: TRadioGroup;
    // buttons...
    btnInfo: TButton;
    btnCopy: TButton;
    // check boxes...
    cbRound: TCheckBox;
    procedure FormShow(Sender: TObject);
    procedure btnInfoClick(Sender: TObject);
    procedure btnCopyClick(Sender: TObject);
    procedure cbRoundClick(Sender: TObject);
    procedure edtRoundKeyPress(Sender: TObject; var Key: Char);
    procedure edtTmpChange(Sender: TObject);
  private
    { Private declarations }
    procedure Calculate;
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

uses untInfo;

{$R *.DFM}

var IsFirstRun: boolean = True;

procedure TfrmMain.FormShow(Sender: TObject);
begin
  edtAWS.Text := '1' + String(DecimalSeparator) + '8';   //1,8
  edtC2.Text  := '10' + String(DecimalSeparator) + '45'; //10,45
  IsFirstRun := False;
end;

procedure TfrmMain.edtTmpChange(Sender: TObject);
begin
  if IsFirstRun then Exit;
  // called from all edit boxes, calculates result
  // after any edit box is changed
  Calculate;
end;

procedure TfrmMain.btnInfoClick(Sender: TObject);
begin
  // show info/help form
  frmInfo.ShowModal;
end;

procedure TfrmMain.btnCopyClick(Sender: TObject);
begin
  // copy result to the clipboard
  Clipboard.AsText := lblResult.Caption;
end;

procedure TfrmMain.cbRoundClick(Sender: TObject);
begin
  // enable rounding and recalculate
  edtRound.Enabled := cbRound.Checked;
  udRound.Enabled  := cbRound.Checked;
  lblRound.Enabled := cbRound.Checked;
  Calculate;
end;

procedure TfrmMain.edtRoundKeyPress(Sender: TObject; var Key: Char);
begin
  // make sure we don't get out-of-bounds with the rounding
  if not (Key in ['0'..'7']) then Key := Chr(0);
end;

procedure TfrmMain.Calculate;
var
  WSpeed, Temp, BaseWSpeed, SkinTemp, Const1, Const2, Rslt, _edtRound: Extended;
  txtC, txtF, txtK, txtR, theTxt, tmpTxt: string;
begin
  WSpeed     := StrToFloat(edtWS.Text);
  if edtTmp.Text <> '-' then Temp := StrToFloat(edtTmp.Text) else
    Temp := 0;
  SkinTemp   := StrToFloat(edtST.Text);
  BaseWSpeed := StrToFloat(edtAWS.Text);
  Const1     := StrToFloat(edtC1.Text);
  Const2     := StrToFloat(edtC2.Text);

  // change the temperature indicator
  txtC := ' ' + #176 + 'C';
  txtF := ' ' + #176 + 'F';
  txtK := ' K';
  txtR := ' ' + #176 + 'R';

  // the transformed Siple formula
  Rslt       := -((Const1 * sqrt(WSpeed) - WSpeed + Const2)*(SkinTemp - Temp)/
    (Const1 * sqrt(BaseWSpeed) - BaseWSpeed + Const2) - SkinTemp);

  case rgTS.ItemIndex of
    0: begin
         theTxt := txtC;
       end;
    1: begin
         // Celcius to Fahrenheit
         Rslt := 9 * Rslt / 5 + 32;
         theTxt := txtF;
       end;
    2: begin
         // Celcius to Kelvin
         Rslt := Rslt + 273.15;
         theTxt := txtK;
       end;
    3: begin
         // Celcius to Raumur
         Rslt := 4 * Rslt / 5;
         theTxt := txtR;
       end;
  end;

  case cbRound.Checked of
    // rounding
    True : begin
             // make sure we don't get out-of-bounds with the rounding,
             // since the Round() function is a longint, the limit is
             // 10 digits.
             _edtRound := StrToFloat(edtRound.Text);
             if _edtRound > 7 then _edtRound := 7;

             // a trick to get a power of 10 function:
             // Exp(2*Ln(10)) = What exponent gives the Ln of 4.605170186?
             // Answer: Ln(100) = 4.605170186, answer is: 100
             tmpTxt := FloatToStr(Round(Rslt * Exp(_edtRound * Ln(10))) /
                         Exp(_edtRound * Ln(10)));
           end;

    // no rounding, just add the temp indicator
    False: tmpTxt := FloatToStr(Rslt);
  end;

  // show result plus temperature indicator
  lblResult.Caption := tmpTxt + theTxt;
end;

end.
